local _G = getfenv(0)
local LibStub = _G.LibStub
local JumpCounter = LibStub("AceAddon-3.0"):NewAddon("JumpCounter", "AceHook-3.0")
_G.JumpCounter = JumpCounter
local db, realm, char
local L = LibStub:GetLibrary("AceLocale-3.0"):GetLocale("JumpCounter")

local GetRealmName = _G.GetRealmName
local UnitName = _G.UnitName
local GetTime = _G.GetTime
local UnitOnTaxi = _G.UnitOnTaxi
local IsFlying = _G.IsFlying
local IsSwimming = _G.IsSwimming
local HasFullControl = _G.HasFullControl
local UnitIsDeadOrGhost = _G.UnitIsDeadOrGhost

local defaults = {
	profile = {
	}
}

local feed = LibStub("LibDataBroker-1.1"):NewDataObject("JumpCounter")
feed.type = "data source"
feed.icon = "Interface/Icons/Inv_Boots_02"
feed.label = "Jumps"
feed.text = "#NaN"

function JumpCounter:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("JumpCounterDB", defaults, true)
	db = self.db.profile
end

function JumpCounter:OnEnable()
	self:SecureHook("JumpOrAscendStart")

	realm = GetRealmName()
	char = UnitName("player")

	if not db[realm] then
		db[realm] = {}
	end

	if not db[realm][char] then
		db[realm][char] = 0
	end

	self.lastJumpTime = GetTime()
	feed.text = db[realm][char]
end

function JumpCounter:OnDisable()
	self:UnhookAll()
end

function JumpCounter:JumpOrAscendStart()
	if UnitOnTaxi("player") or IsFlying() or IsSwimming() or HasFullControl() == false or UnitIsDeadOrGhost("player") then return end
	local now = GetTime()
	if now - self.lastJumpTime >= 0.8 then
		self.lastJumpTime = now
		db[realm][char]= db[realm][char]+ 1
		feed.text = db[realm][char]
		sort(db[realm], function(a,b) return a < b end)
	end
end

local function GetTipAnchor(frame)
	local x,y = frame:GetCenter()
	if not x or not y then return "TOPLEFT", "BOTTOMLEFT" end
	local hhalf = (x > UIParent:GetWidth()*2/3) and "RIGHT" or (x < UIParent:GetWidth()/3) and "LEFT" or ""
	local vhalf = (y > UIParent:GetHeight()/2) and "TOP" or "BOTTOM"
	return vhalf..hhalf, frame, (vhalf == "TOP" and "BOTTOM" or "TOP")..hhalf
end

function feed.OnLeave() GameTooltip:Hide() end
function feed.OnEnter(self)
 	GameTooltip:SetOwner(self, "ANCHOR_NONE")
	GameTooltip:SetPoint(GetTipAnchor(self))
	GameTooltip:ClearLines()
	GameTooltip:AddLine("JumpCounter")
	local total = 0
	for realm, chars in pairs(db) do
		for char, jumps in pairs(chars) do
			GameTooltip:AddDoubleLine(string.format("%s - %s",char, realm), jumps)
			total = total + jumps
		end
	end
	GameTooltip:AddLine(" ")
	GameTooltip:AddDoubleLine("Total", total)
	GameTooltip:Show()
end
